/* jshint undef: true */
/* global plugin, require, com, java */

var _ = require("lodash");
var on = require("on")(plugin);
var Objects = require("convenience").Objects;
var facade = require("facade");
var File = java.io.File;
var EventType = com.cattura.agents.configuration_module.model.vos.events.EventType;
var StringUtilities = com.cattura.cattura_commons.utilities.StringUtilities;
var STORE = plugin.info().dataStore;
var LOGGER = plugin.info().logger;
var oauths = facade.retrieveProxy("OAuthProxy");

function Notifier (data)
{
    _(this).extend(data).value();
    this.id = this.id || this.token;
    
    this.canHandleEvent = function canHandleEvent (event)
    {
        if (event.type.category == EventType.Category.INTERNAL)
        {
            return false;
        }
        
        return _(this.events).find(function (type) { 
            return type == event.type.name() || type == event.getClass().simpleName;
        });
    };
    
    this.handleEvent = function handleEvent (event)
    {
        var login = oauths.getCredentialsForServiceWithToken("twitter", this.token).orNull();
        if (login && login.valid)
        {
            try
            {
                login.client.updateStatus(event.getDescription());                    
            }
            catch (e)
            {
                LOGGER.info(e.message);
            }
        }
    };
}

function migrateLegacyTokens ()
{
    if (!STORE.has("tokens"))
    {
        return;
    }
    
    var events = Objects.toJS(STORE.get("event-types").orElse([]), true);
    _(Objects.toJS(STORE.get("tokens").get(), true)).each(function (token) {
        addNotifier(token, token, events);
    });
    STORE.remove("tokens");
    STORE.remove("event-types");
    LOGGER.info("Migrated legacy twitter notification configuration");
}

function getNotifiers ()
{
    return _(Objects.toJS(STORE.select("notifier:*"))).map(function (notifier) {
        return new Notifier(notifier);
    }).value();
}

function addNotifier (id, token, events)
{
    STORE.set("notifier:" + id, {id: id, token: token, events: events});
}

function removeNotifier (id)
{
    STORE.remove("notifier:" + id);
}

function getNotifiersForEvent (event)
{
    return _(getNotifiers()).filter(function (notifier) {
        return notifier.canHandleEvent(event);
    }).value();
}

function generateIdForNotifier ()
{
    var id = StringUtilities.createRandomString(32, "abcdefghijklmnopqrstuvwxyz0123456789");
    if (_(getNotifiers()).find(function (notifier) { return notifier.id === id; }))
    {
        return generateIdForNotifier();
    }
    
    return id;
}

function getExceptionFromEvent (event)
{
    if ('incident' in event)
    {
        return event.incident.exception;
    }
    else if ('exception' in event)
    {
        return event.exception;
    }
}

plugin.service({
    name: "addNotifier",
    description: "Add a notifier",
    parameters: [
        {name: "token", description: "Twitter token", type: "java.lang.String"},
        {name: "events", description: "Events to notify on", type: "java.util.List"}
    ],
    handler: function (token, events) {
        addNotifier(generateIdForNotifier(), token, events);
    }
});

plugin.service({
    name: "addNotifier",
    description: "Add a notifier",
    parameters: [
        {name: "id", description: "Unique identifier for notifier", type: "java.lang.String"},
        {name: "token", description: "Twitter token", type: "java.lang.String"},
        {name: "events", description: "Events to notify on", type: "java.util.List"}
    ],
    handler: addNotifier
});

plugin.service({
    name: "removeNotifier",
    description: "Remove a notifier",
    parameters: [
        {name: "id", description: "Unique identifier for notifier", type: "java.lang.String"}
    ],
    handler: removeNotifier
});

plugin.service({
    name: "getNotifiers",
    description: "Return all registered notifiers",
    parameters: [],
    handler: getNotifiers
});

/*
 * Migrate legacy handlers to new system
 */
migrateLegacyTokens();

plugin.onEvent(function(event) {
    try
    {
        _(getNotifiersForEvent(event)).each(function (notifier) {
            notifier.handleEvent(event);
        });
    }
    catch (exception)
    {
        if (getExceptionFromEvent(event))
        {
            LOGGER.info("Twitter Plugin encountered (and swallowed) exception while trying to handle exception event");
            return;
        }
        
        throw new java.lang.RuntimeException("Twitter Plugin encountered exception while trying to handle event: " + event, exception.javaException);
    }
});